/*
 * Fits.c
 *
 * by John Pavan
 *
 * Functions for use with gnuscope
 * which are similar to fits or
 * help when using fits
 */

#include <gtk/gtk.h>
#include "gtkgraph.h"
#include <math.h>
#include <stdio.h>
#include "fits.h"
#include "gnuscopeglobals.h"

/* --- Globals --- */
//int histid[254],histsize[254],histpointer;
//int *histloc[254];
//char field1[254][40],field2[254][40],field3[254][40];
int intype,spectra,plottype;
GtkWidget *graph;
int markers[4],binsize,offset,currentrange[2];
float scaling;
int yscale,ybin,yscaleforce,ybinforce;
long double displayused;
int numspectra, spectradisplayed[16];
GtkWidget *box1,*graphsdisplayed[16];
int displaygaussian,displaydoublegaussian;
GtkWidget *maintext;
struct stoppingpower *stoppingpowerinfo;
GtkWidget *population[10][4];
GtkWidget *tauentry[10][4];
GtkWidget *tsfentry[10][4];
struct fitsvariables *fitsstuff;

/* --- Function Declarations --- */

void GetMessageDialog(const char *message);
void WriteMainText(const char *newtext);

void FitsReadStoppingPower(char *sFilename);
GtkWidget *MakeFeedingEntry(int level, int pos);
void FeedingFileSaveCallback(GtkWidget *widget, gpointer *data);
void FeedingFileSave(char *sFilename);
void FeedingFileReadCallback(GtkWidget *widget, gpointer *data);
void FeedingFileRead(char *sFilename);
void FeedingClear(GtkWidget *widget, gpointer *data);
void ReadFitsSetup(char *sFilename);
void Fits();


/* --- functions --- */

/* ListBetweenMarkers
 *
 * lists the counts in the channels between (and including) the markers
 */
void ListBetweenMarkers()
{
  int i;
  int localmarkers[2];
  char dummystr[80];

  /* --- get the markers in order --- */
  if (markers[0] > markers[1]) {
    localmarkers[0] = markers[1];
    localmarkers[1] = markers[0];
  } else {
    localmarkers[1] = markers[1];
    localmarkers[0] = markers[0];
  }

  for (i = localmarkers[1] ; i >= localmarkers[0];i--) {
    sprintf(dummystr,"%d ",*(histloc[spectra] + i));
    WriteMainText(dummystr);
    if ((i % 10) == 0) {
      WriteMainText("\n");
    }
  }
  sprintf(dummystr,"Counts between channels %d and %d.\n",(localmarkers[0]+1),(localmarkers[1]+1));
  WriteMainText(dummystr);
}

/* FitsReadStoppingPower
 *
 * Reads in a Stopping Power file of the same format as fits.
 * Details of what is being read in will be included in
 * comments of the code.
 */
void FitsReadStoppingPower(char *sFilename)
{
  int i,j,k,l,m;
  FILE *infile;
  char dummystr[160];
  float a,b,c,d,e,f,g,h;
  char *charpointer;

  if ((infile = fopen(sFilename,"r")) != NULL) {
    /* --- first if stoppingpowerinfo has not already been mallocated, we need to
       --- mallocate it --- */
    if (stoppingpowerinfo == NULL) {
      stoppingpowerinfo = (struct stoppingpower *) malloc(sizeof(struct stoppingpower));
      stoppingpowerinfo->rectheta = NULL;
      stoppingpowerinfo->recphi = NULL;
      stoppingpowerinfo->recprobtheta = NULL;
      stoppingpowerinfo->projstop = NULL;
      stoppingpowerinfo->stopen1 = NULL;
      stoppingpowerinfo->stop1 = NULL;
      stoppingpowerinfo->stopen2 = NULL;
      stoppingpowerinfo->stop2 = NULL;
      stoppingpowerinfo->crosssectionen = NULL;
      stoppingpowerinfo->crosssection = NULL;
    }
    /* --- skip the first line, we won't be using it --- */
    /* --- the first line is a comment about what the file contains --- */
    fgets(dummystr,80,infile);
    /* --- now read in the projectile mass and energy --- */
    if (fscanf(infile,"%f,%f",&a,&b) == 2) {
      stoppingpowerinfo->projmass = a;
      stoppingpowerinfo->beamenergy = b;
    } else {
      GetMessageDialog("Error Reading the projectile mass and beam energy.\n");
      goto stopreading;
    }
    /* --- get the target a,z and the backing a,z --- */
    if (fscanf(infile,"%d,%d,%d,%d",&i,&j,&k,&l) == 4) {
      stoppingpowerinfo->targa = i;
      stoppingpowerinfo->targz = j;
      stoppingpowerinfo->backa = k;
      stoppingpowerinfo->backz = l;
    } else {
      GetMessageDialog("Error Reading the target A,Z and the backing A,Z.\n");
      goto stopreading;
    }
    /* --- now for:
       --- blaugrund's approx switch
       --- fractional FWHM of recoil energy distribution
       --- number of recoil theta angles
       --- number of recoil phi angles (why?  They arn't used)
       --- */
    if (fscanf(infile,"%d,%d,%d,%f,%d,%d",&i,&j,&k,&a,&l,&m) == 6) {
      stoppingpowerinfo->reca = i;
      stoppingpowerinfo->recz = j;
      stoppingpowerinfo->blauswitch = k;
      stoppingpowerinfo->recfwhmen = a;
      stoppingpowerinfo->numrectheta = l;
      stoppingpowerinfo->numrecphi = m;
    } else {
      GetMessageDialog("Error Reading recoil information.\n");
      goto stopreading;
    }
    /* --- now we need to read in the recoil thetas and probablilities --- */
    /* --- first get the right amount of memory --- */
    if (stoppingpowerinfo->rectheta == NULL) {
      stoppingpowerinfo->rectheta = (float *)
	malloc(stoppingpowerinfo->numrectheta * sizeof(float));
    } else {
      stoppingpowerinfo->rectheta = (float *)
	realloc(stoppingpowerinfo->rectheta, stoppingpowerinfo->numrectheta * sizeof(float));
    }
    if (stoppingpowerinfo->recprobtheta == NULL) {
      stoppingpowerinfo->recprobtheta = (float *) 
	malloc(stoppingpowerinfo->numrectheta * sizeof(float));
    } else {
      stoppingpowerinfo->recprobtheta = (float *)
	realloc(stoppingpowerinfo->recprobtheta, stoppingpowerinfo->numrectheta * sizeof(float));
    }
    /* --- now let's read the info --- */
    for (i = 0; i < stoppingpowerinfo->numrectheta; i++) {
      if (fscanf(infile,"%f,%f",&a,&b) == 2) {
	stoppingpowerinfo->rectheta[i] = a;
	stoppingpowerinfo->recprobtheta[i] = b;
      } else {
	GetMessageDialog("Error reading recoil angles.");
	goto stopreading;
      }
    }
    /* --- now let's read the detector angles/resolutions --- */
    if (fscanf(infile,"%f,%f,%f",&a,&b,&c) == 3) {
      stoppingpowerinfo->detres[0] = a;
      stoppingpowerinfo->detres[1] = b;
      stoppingpowerinfo->detres[2] = c;
    } else {
      GetMessageDialog("Error Reading the detector resolutions.");
      goto stopreading;
    }
    /* --- now we need to read:
       --- the projectile stopping power normalization
       --- the minium beam energy
       --- the energy increment for projectile stopping powers
       --- the number of projectiles stopping powers
       --- */
    if (fscanf(infile,"%f,%f,%f,%d",&a,&b,&c,&i) == 4) {
      stoppingpowerinfo->projstopnorm = a;
      stoppingpowerinfo->minbeamen = b;
      stoppingpowerinfo->debeamen = c;
      stoppingpowerinfo->numprojstop = i;
    } else {
      GetMessageDialog("Error reading projectile information.");
      goto stopreading;
    }
    /* --- now we need to read in the projectile stopping powers --- */
    /* --- first we need to allocate enough memory --- */
    if (stoppingpowerinfo->projstop == NULL) {
      stoppingpowerinfo->projstop = 
	(float *) malloc(stoppingpowerinfo->numprojstop * sizeof(float));
    } else {
      stoppingpowerinfo->projstop = 
	(float *) realloc(stoppingpowerinfo->projstop, 
			  stoppingpowerinfo->numprojstop * sizeof(float));
    }
    /* --- now let's read it in --- */
    /* --- this is going to be interesting because they tend to be written
       --- in one row. --- */
    i = 0;
    //    printf("number of projectile stopping powers %d.\n",
    //	   stoppingpowerinfo->numprojstop);
      fgets(dummystr,160,infile);
    while (i < stoppingpowerinfo->numprojstop) {
      /* --- read in a line --- */
      //printf("about to read line.\n");
      if (fgets(dummystr,160,infile) == NULL) {
	GetMessageDialog("Error reading projectile stopping powers.");
      	goto stopreading;
      }
      //printf("read line.\n");
      /* --- now we need to extract the information from that string --- */
      charpointer = (char *) &dummystr;
      //printf(dummystr);
      //printf(charpointer);
      while ((charpointer != NULL) && (i < stoppingpowerinfo->numprojstop)) {
	if (sscanf(charpointer,"%f",&a) == 1) {
	  stoppingpowerinfo->projstop[i] = a;
	  //printf("%f\n",a);
	  i++;
	  //printf("about to get a new pointer.\n");
	  charpointer = (char *) strstr(charpointer,",");
	  if (charpointer != NULL) {
	    charpointer = &charpointer[1];
	    // printf(charpointer);
	  }
	  //printf("got a new pointer.\n");
	} else {
	  GetMessageDialog("Error reading projectile stopping powers.");
	  goto stopreading;
	}
      }
    }
    /* --- now we need to read in:
       --- the number of target elements,
       --- the element thickness,
       --- the number of coarse stopping powers
       --- the stopping power normalization,
       --- the number of coarse stoping powers to read
       --- and the number of recoils to start with.
       --- */

    if (fscanf(infile,"%d,%f,%d,%f,%d,%f,%d",&i,&a,&k,&b,&l,&c,&m) == 7) {
      stoppingpowerinfo->numtarel = i;
      stoppingpowerinfo->tarelthick = a;
      stoppingpowerinfo->numcoarsestop = k;
      stoppingpowerinfo->tarstopnorm[0] = b;
      stoppingpowerinfo->numrecoils[0] = l;
      stoppingpowerinfo->tarstopnorm[1] = c;
      stoppingpowerinfo->numrecoils[1] = m;
    } else {
      GetMessageDialog("Error Reading the stopping power initialization.\n");
      goto stopreading;
    }
    /* --- now we need to read some floats that I don't know what to do with --- */

    if (fscanf(infile,"%f,%f,%f,%f,%d",&a,&b,&c,&d,&i) == 5) {
      stoppingpowerinfo->fe[0] = a;
      stoppingpowerinfo->fn[0] = b;
      stoppingpowerinfo->fe[1] = c;
      stoppingpowerinfo->fn[1] = d;
      stoppingpowerinfo->nchoice = i;
    } else {
      GetMessageDialog("Error reading stopping power file.\n");
      goto stopreading;
    }

    /* --- now comes the fun part, we need to read in the stopping powers
       --- but first we need to get memory for them --- */
    if (stoppingpowerinfo->stopen1 == NULL) {
      stoppingpowerinfo->stopen1 = (float *) 
	malloc(sizeof(float) * stoppingpowerinfo->numcoarsestop);
    } else {
      stoppingpowerinfo->stopen1 = (float *) 
	realloc(stoppingpowerinfo->stopen1,sizeof(float) * stoppingpowerinfo->numcoarsestop);
    }
    if (stoppingpowerinfo->stop1 == NULL) {
      stoppingpowerinfo->stop1 = (float *)
	malloc(sizeof(float) * stoppingpowerinfo->numcoarsestop);
    } else {
      stoppingpowerinfo->stop1 = (float *)
	realloc(stoppingpowerinfo->stop1,sizeof(float) * stoppingpowerinfo->numcoarsestop);
    }
    if (stoppingpowerinfo->stopen2 == NULL) {
      stoppingpowerinfo->stopen2 = (float *) 
	malloc(sizeof(float) * stoppingpowerinfo->numcoarsestop);
    } else {
      stoppingpowerinfo->stopen2 = (float *) 
	realloc(stoppingpowerinfo->stopen2,sizeof(float) * stoppingpowerinfo->numcoarsestop);
    }
    if (stoppingpowerinfo->stop2 == NULL) {
      stoppingpowerinfo->stop2 = (float *)
	malloc(sizeof(float) * stoppingpowerinfo->numcoarsestop);
    } else {
      stoppingpowerinfo->stop2 = (float *)
	realloc(stoppingpowerinfo->stop2,sizeof(float) * stoppingpowerinfo->numcoarsestop);
    }
    /* --- now that memory is allocated let's get the information --- */
    for (i = 0; i < stoppingpowerinfo->numcoarsestop; i++) {
      if (fscanf(infile,"%f,%f,%f,%f",&a,&b,&c,&d) == 4) {
	stoppingpowerinfo->stopen1[i] = a;
	stoppingpowerinfo->stop1[i] = b;
	stoppingpowerinfo->stopen2[i] = c;
	stoppingpowerinfo->stop2[i] = d;
      } else {
	GetMessageDialog("Error reading stopping powers.\n");
	goto stopreading;
      }
    }
    /* --- now we need to read in the number of residual nucleus production cross-sections --- */
    if (fscanf(infile,"%d",&i) == 1) {
      stoppingpowerinfo->numresidprodcross = i;
    } else {
      GetMessageDialog("Error reading the number of residual nucleus production crosssections.");
      goto stopreading;
    }
    /* --- must allocate memory for the cross section information --- */
    if (stoppingpowerinfo->crosssectionen == NULL) {
      stoppingpowerinfo->crosssectionen = (float *) 
	malloc(sizeof(float) * stoppingpowerinfo->numresidprodcross);
    } else {
      stoppingpowerinfo->crosssectionen = (float *) 
	realloc(stoppingpowerinfo->crosssectionen,
		sizeof(float) * stoppingpowerinfo->numresidprodcross);
    }
    if (stoppingpowerinfo->crosssection == NULL) {
      stoppingpowerinfo->crosssection = (float *) 
	malloc(sizeof(float) * stoppingpowerinfo->numresidprodcross);
    } else {
      stoppingpowerinfo->crosssection = (float *) 
	realloc(stoppingpowerinfo->crosssection,
		sizeof(float) * stoppingpowerinfo->numresidprodcross);
    }

    /* --- now we need to read in the cross-section tables and corresponding energies --- */
    i = 0;
    fgets(dummystr,160,infile);
    while (i < stoppingpowerinfo->numresidprodcross) {
      if (fgets(dummystr,160,infile) == NULL) {
	GetMessageDialog("Error reading residual cross section energies.\n");
	goto stopreading;
      }
      charpointer = (char *) &dummystr;
      while ((charpointer != NULL) && (i < stoppingpowerinfo->numresidprodcross)) {
	if (sscanf(charpointer,"%f",&a) == 1) {
	  stoppingpowerinfo->crosssectionen[i] = a;
	  i++;
	  //printf("%f\n",a);
	  charpointer = (char *) strstr(charpointer,",");
	  if (charpointer != NULL) 
	    charpointer = (char *) &charpointer[1];
	} else {
	  GetMessageDialog("Error reading the residual cross section energies.\n");
	}
      }
    }    
    i = 0;
    while (i < stoppingpowerinfo->numresidprodcross) {
      if (fgets(dummystr,160,infile) == NULL) {
	GetMessageDialog("Error reading residual cross section energies.\n");
	goto stopreading;
      }
      charpointer = (char *) &dummystr;
      while ((charpointer != NULL) && (i < stoppingpowerinfo->numresidprodcross)) {
	if (sscanf(charpointer,"%f",&a) == 1) {
	  stoppingpowerinfo->crosssection[i] = a;
	  //printf("%f\n",a);
	  i++;
	  charpointer = (char *) strstr(charpointer,",");
	  if (charpointer != NULL) 
	    charpointer = (char *) &charpointer[1];
	} else {
	  GetMessageDialog("Error reading the residual cross section energies.\n");
	}
      }
    }
    stoppingpowerinfo->tardensity = stoppingpowerinfo->tarelthick / (float)
      stoppingpowerinfo->numtarel;

  stopreading:
    fclose(infile);
  }
}

/* MakeFeedingEntry
 *
 * Produces a frame with three entries in it for population, tauentry, and tsfentry
 * according to the level and position fed to it.
 */
GtkWidget *MakeFeedingEntry(int level, int pos)
{
  GtkWidget *localframe;
  GtkWidget *localhbox;
  GtkWidget *locallabel;
  char dummystr[40];
  
  sprintf(dummystr,"%d,%d",level+1,pos+1);
  localframe = gtk_frame_new(dummystr);

  localhbox = gtk_hbox_new(FALSE,0);
  gtk_container_add(GTK_CONTAINER(localframe),localhbox);

  locallabel = gtk_label_new("Pop = ");
  gtk_container_add(GTK_CONTAINER(localhbox),locallabel);
  population[level][pos] = gtk_entry_new();
  gtk_widget_set_usize(population[level][pos],25,22);
  gtk_container_add(GTK_CONTAINER(localhbox),population[level][pos]);
  locallabel = gtk_label_new("  tauentry = ");
  gtk_container_add(GTK_CONTAINER(localhbox),locallabel);
  tauentry[level][pos] = gtk_entry_new();
  gtk_widget_set_usize(tauentry[level][pos],25,22);
  gtk_container_add(GTK_CONTAINER(localhbox),tauentry[level][pos]);
  locallabel = gtk_label_new("ps  tauentry sf = ");
  gtk_container_add(GTK_CONTAINER(localhbox),locallabel);
  tsfentry[level][pos] = gtk_entry_new();
  gtk_widget_set_usize(tsfentry[level][pos],25,22);
  gtk_container_add(GTK_CONTAINER(localhbox),tsfentry[level][pos]);
  locallabel = gtk_label_new("ps");
  gtk_container_add(GTK_CONTAINER(localhbox),locallabel);

  return(localframe);
}

/* OpenFeedingWindow
 *
 * Creates a window to enter feeding information into
 */
void OpenFeedingWindow()
{
  GtkWidget *localwindow;
  GtkWidget *localtable;
  GtkWidget *localframe;
  GtkWidget *localbutton;
  int i,j;

  for (i = 0; i < 10; i++) {
    for (j = 0; j < 4; j++) {
      population[i][j] = NULL;
      tauentry[i][j] = NULL;
      tsfentry[i][j] = NULL;
    }
  }

  localwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(localwindow),"Feeding Information");
  
  localtable = gtk_table_new(5,10,FALSE);
  gtk_container_add(GTK_CONTAINER(localwindow),localtable);
  gtk_table_set_row_spacing(GTK_TABLE(localtable),0,2);
  gtk_table_set_col_spacing(GTK_TABLE(localtable),2,0);
  localframe = MakeFeedingEntry(0,0);
  gtk_table_attach(GTK_TABLE(localtable),localframe,2,3,9,10,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(1,0);
  gtk_table_attach(GTK_TABLE(localtable),localframe,0,2,8,9,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(1,1);
  gtk_table_attach(GTK_TABLE(localtable),localframe,3,5,8,9,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(2,0);
  gtk_table_attach(GTK_TABLE(localtable),localframe,0,1,7,8,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(2,1);
  gtk_table_attach(GTK_TABLE(localtable),localframe,1,2,7,8,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(2,2);
  gtk_table_attach(GTK_TABLE(localtable),localframe,3,4,7,8,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(2,3);
  gtk_table_attach(GTK_TABLE(localtable),localframe,4,5,7,8,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(3,0);
  gtk_table_attach(GTK_TABLE(localtable),localframe,0,1,6,7,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(3,1);
  gtk_table_attach(GTK_TABLE(localtable),localframe,1,2,6,7,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(3,2);
  gtk_table_attach(GTK_TABLE(localtable),localframe,3,4,6,7,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(3,3);
  gtk_table_attach(GTK_TABLE(localtable),localframe,4,5,6,7,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(4,0);
  gtk_table_attach(GTK_TABLE(localtable),localframe,0,1,5,6,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(4,1);
  gtk_table_attach(GTK_TABLE(localtable),localframe,1,2,5,6,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(4,2);
  gtk_table_attach(GTK_TABLE(localtable),localframe,3,4,5,6,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(4,3);
  gtk_table_attach(GTK_TABLE(localtable),localframe,4,5,5,6,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(5,0);
  gtk_table_attach(GTK_TABLE(localtable),localframe,0,1,4,5,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(5,1);
  gtk_table_attach(GTK_TABLE(localtable),localframe,1,2,4,5,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(5,2);
  gtk_table_attach(GTK_TABLE(localtable),localframe,3,4,4,5,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localframe = MakeFeedingEntry(5,3);
  gtk_table_attach(GTK_TABLE(localtable),localframe,4,5,4,5,
		   GTK_EXPAND | GTK_SHRINK,
		   GTK_EXPAND | GTK_SHRINK,
		   0,0);
  localbutton = gtk_button_new_with_label("Read Feeding File");
  gtk_table_attach(GTK_TABLE(localtable),localbutton,0,1,0,1,
		   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		   0,0);
  gtk_signal_connect(GTK_OBJECT(localbutton),"clicked",
		     GTK_SIGNAL_FUNC(FeedingFileReadCallback),
		     NULL);
  localbutton = gtk_button_new_with_label("Write Feeding File");
  gtk_table_attach(GTK_TABLE(localtable),localbutton,1,2,0,1,
		   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		   0,0);
  gtk_signal_connect(GTK_OBJECT(localbutton),"clicked",
		     GTK_SIGNAL_FUNC(FeedingFileSaveCallback),
		     NULL);
  localbutton = gtk_button_new_with_label("Clear");
  gtk_table_attach(GTK_TABLE(localtable),localbutton,2,3,0,1,
		   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		   0,0);
  gtk_signal_connect(GTK_OBJECT(localbutton),"clicked",
		     GTK_SIGNAL_FUNC(FeedingClear),
		     NULL);
  
  
  gtk_widget_show_all(localwindow);
}

/* FeedingFileSaveCallback
 *
 * Callback function which saves the current feeding information
 */
void FeedingFileSaveCallback(GtkWidget *widget, gpointer *data)
{
  GetFilename("Save Feeding Information",FeedingFileSave);
}


/* FeedingFileSave
 *
 * Saves the current feeding information
 */
void FeedingFileSave(char *sFilename)
{
  FILE *outfile;
  char dummystr[80];
  int i,j,k,l,m;
  float a,b,c,d,e;
  float popout,sfout,tauentryout;

  if ((outfile = fopen(sFilename,"w")) != NULL) {
    for (i = 0; i < 10; i++) {
      for (j = 0; j < 4; j++) {
	if ((population[i][j] != NULL) || (tauentry[i][j] != NULL) || (tsfentry[i][j] != NULL)) {
	  if (sscanf(gtk_entry_get_text(GTK_ENTRY(population[i][j])),"%f",&a) == 1) {
	    popout = a;
	  } else {
	    popout = 0;
	  }
	  if (sscanf(gtk_entry_get_text(GTK_ENTRY(tauentry[i][j])),"%f",&a) == 1) {
	    tauentryout = a;
	  } else {
	    tauentryout = 0;
	  }
	  if (sscanf(gtk_entry_get_text(GTK_ENTRY(tsfentry[i][j])),"%f",&a) == 1) {
	    sfout = a;
	  } else {
	    sfout = 0;
	  }
	  if ((popout != 0) || (tauentryout != 0) || (sfout != 0))
	    fprintf(outfile,"%d,%d,%f,%f,%f\n",i,j,popout,sfout,tauentryout);
	}
      }
    }

    fclose(outfile);
  }
}

/* FeedingFileReadCallback
 *
 * Callback function which saves the current feeding information
 */
void FeedingFileReadCallback(GtkWidget *widget, gpointer *data)
{
  GetFilename("Read Feeding Information",FeedingFileRead);
}


/* FeedingFileRead
 *
 * Saves the current feeding information
 */
void FeedingFileRead(char *sFilename)
{
  FILE *infile;
  char dummystr[80];
  int i,j,k,l,m;
  float a,b,c,d,e;
  float popout,sfout,tauentryout;

  if ((infile = fopen(sFilename,"r")) != NULL) {

    while(fgets(dummystr,80,infile) != NULL) {
      if(sscanf(dummystr,"%d,%d,%f,%f,%f",&i,&j,&a,&b,&c) == 5) {
	if (population[i][j] != NULL) {
	  sprintf(dummystr,"%f",a);
	  gtk_entry_set_text(GTK_ENTRY(population[i][j]),dummystr);
	}
	if (tsfentry[i][j] != NULL) {
	  sprintf(dummystr,"%f",b);
	  gtk_entry_set_text(GTK_ENTRY(tsfentry[i][j]),dummystr);
	}
	if (tauentry[i][j] != NULL) {
	  sprintf(dummystr,"%f",c);
	  gtk_entry_set_text(GTK_ENTRY(tauentry[i][j]),dummystr);
	}

      }
    }

    fclose(infile);
  }
}

/* FeedingClear
 *
 * Callback function which saves the current feeding information
 */
void FeedingClear(GtkWidget *widget, gpointer *data)
{
  int i,j,k;

  for (i = 0; i < 10; i++) {
    for (j = 0; j < 4; j++) {
      if (population[i][j] != NULL)
	gtk_entry_set_text(GTK_ENTRY(population[i][j]),"");
      if (tauentry[i][j] != NULL)
	gtk_entry_set_text(GTK_ENTRY(tauentry[i][j]),"");
      if (tsfentry[i][j] != NULL)
	gtk_entry_set_text(GTK_ENTRY(tsfentry[i][j]),"");
    }
  }
}

/* ReadFitsSetup
 *
 * Reads the last little bit of information needed to do a fits calculation
 */
void ReadFitsSetup(char *sFilename)
{
  int i,j,k;
  FILE *infile;
  float a,b,c;
    
  if ((infile = fopen(sFilename,"r")) != NULL) {
    
    if (stoppingpowerinfo == NULL) {
      stoppingpowerinfo = (struct stoppingpower *) malloc(sizeof(struct stoppingpower));
      stoppingpowerinfo->rectheta = NULL;
      stoppingpowerinfo->recphi = NULL;
      stoppingpowerinfo->recprobtheta = NULL;
      stoppingpowerinfo->projstop = NULL;
      stoppingpowerinfo->stopen1 = NULL;
      stoppingpowerinfo->stop1 = NULL;
      stoppingpowerinfo->stopen2 = NULL;
      stoppingpowerinfo->stop2 = NULL;
      stoppingpowerinfo->crosssectionen = NULL;
      stoppingpowerinfo->crosssection = NULL;
    }

    if (fscanf(infile,"%f,%f,%f",&a,&b,&c) == 3) {
      stoppingpowerinfo->theta = a;
      stoppingpowerinfo->a2 = b;
      stoppingpowerinfo->a4 = c;
    } else {
      GetMessageDialog("Error reading detector angle information.\n");
      goto stopreading;
    }
    if (fscanf(infile,"%f,%f,%f,%d,%d",&a,&b,&c,&i,&j) == 5) {
      stoppingpowerinfo->r = a;
      stoppingpowerinfo->h = b;
      stoppingpowerinfo->d = c;
      stoppingpowerinfo->nrad = i;
      stoppingpowerinfo->nang = j;
    } else {
      GetMessageDialog("Error reading detector geometry information.");
      goto stopreading;
    }

  stopreading:
    fclose(infile);
  }

}

/* Fit
 *
 * Does the fit according to the old system.  For more details see fits.f
 * in ~tabor/dsam/.
 */
void Fit()
{
  /* --- local variables --- */

  int i,j,k,l,m,n;
  float a,b,c,d,e,f;
  float imax,jmax;

  /* --- allocate memory --- */

  fitsstuff = (struct fitsvariables *) malloc(sizeof(struct fitsvariables));

  /* --- function --- */

  /* --- we have hopefully already read in the stoppng power file --- */
  /* --- guess we can check --- */
  g_return_if_fail(stoppingpowerinfo != NULL);
  /* --- we will also need to have calibrations set
     --- the UseCalibration function returns -1 if there are no
     --- calibrations --- */
  g_return_if_fail(UseCalibration(markers[0]) != -1);

  /* --- now we need to get the experimental spectrum from the
     --- histogram values between the markers (inclusive) --- */
  qsort(&markers,2,sizeof(int),CompareInts); // get the first to markers in the correct order
  fitsstuff->ichn = markers[1] - markers[0] + 1;
  for ( i = 0; i < fitsstuff->ichn; i++) {
    fitsstuff->ispece[i] = (float) *(histloc[spectra] + markers[0] + i);
  }
  /* --- if there is information for efficiency correction
     --- we can use it --- */
  if (UseEfficiency(fitsstuff->ispece[0],UseCalibration(markers[0])) != -1) {
    for (i = 0; i < fitsstuff->ichn; i++) {
      fitsstuff->ispece[i] = UseEfficiency(fitsstuff->ispece[i],UseCalibration(markers[0] + i));
    }
  }
  /* --- there now we have the correct histogram information --- */

  imax = jmax = 0;

  /* --- now we need to get the population parameters --- */
  for ( i = 0; i < 10; i ++) {
    for ( j = 0; j < 4; j ++) {
      if ((population[i][j] != NULL) || (tauentry[i][j] != NULL) || (tsfentry[i][j] != NULL)) {
	if (sscanf(gtk_entry_get_text(GTK_ENTRY(population[i][j])),"%f",&a) == 1) {
	  fitsstuff->pop[i][j] = a;
	} else {
	  fitsstuff->pop[i][j] = 0;
	}
	if (sscanf(gtk_entry_get_text(GTK_ENTRY(tauentry[i][j])),"%f",&a) == 1) {
	  fitsstuff->tau[i][j] = a;
	} else {
	  fitsstuff->tau[i][j] = 0;
	}
	if (sscanf(gtk_entry_get_text(GTK_ENTRY(tsfentry[i][j])),"%f",&a) == 1) {
	  fitsstuff->tsf[i][j] = a;
	} else {
	  fitsstuff->tsf[i][j] = 0;
	}
	if ((popout != 0) || (tauentryout != 0) || (sfout != 0)) {
	  imax = i;
	  jmax = j;
	}
      } else {
	fitsstuff->pop[i][j] = 0;
	fitsstuff->tau[i][j] = 0;
	fitsstuff->tsf[i][j] = 0;
      }
      fitsstuff->tsf[i][j] = fitsstuff->tsf[i][j] * pow(10,-12);
      fitsstuff->tau[i][j] = fitsstuff->tau[i][j] * pow(10,-12);
    } // end j loop
  } // end i loop

  for (i = 0; i < 30; i++)
    fitsstuff->taul[i] = 0;

  /* --- now we have the population parameters --- */

  fitsstuff->igmax = 1;
  fitsstuff->igam = 0;

  /* --- for now we will prompt for the range of tau --- */
  
  GetDialog(100);

  fitsstuff->nlt = (int) 
    ((float) (fitsstuff->tauup - fitsstuff->taulo) / (float) (fitsstuff->dtau) + 1);
  if (fitsstuff->nlt > 29) fitsstuff->nlt = 29;
  fitsstuff->nlts = fitsstuff->nlt + 1;
  for ( i = 0; i < fitsstuff->nlts; i++) {
    j = i - 1;
    fitsstuff->taul[i] = (fitsstuff->taulo + j * fitsstuff->dtau) * pow(10,-12);
  }

  /* --- now we need to get the stop peak info and egam --- */
  
  qsort(&markers[2],2,sizeof(int),CompareInts);
  /* --- later the egam variable should be modified to
     --- vary with stop peak possition --- */
  fitsstuff->egam = UseCalibration(markers[2]);
  
  fitsstuff->pklo = markers[2];
  fitsstuff->pkhi = markers[3];
  fitsstuff->dpeak = (float) (markers[3] - markers[2]) / (float) 9;
  fitsstuff->np1 = (int) 
    (float) (fitsstuff->pkhi - fitsstuff->pkhi) / (float) (fitsstuff->dpeak + 1);
  if (fitsstuff->np1 > 10) np1 = 10;
  fitsstuff->pkhi = (fitsstuff->np1 - 1) * fitsstuff->dpeak + fitsstuff->pklo;

  /* --- get the background and the calibration from the globals --- */
  fitsstuff->ibkg = background;
  if(calibration[spectra][1] != 0) {
    fitsstuff->ecal = calibration[spectra][1];
  } else {
    fitsstuff->ecal = globalcalibration[1];
  }
  
  /* --- while stoppingpowerinfo->nrad and nang are already in memory 
     --- we need to keep them in appropriate ranges --- */
  if (stoppingpowerinfo->nrad > 10) stoppingpowerinfo->nrad = 10;
  if (stoppingpowerinfo->nang > 10) stoppingpowerinfo->nang = 10;

  /* --- now let's actually do the lineshape fitting --- */

  sumg = 0;
  reso = stoppingpowerinfo->detres[0] +
    stoppingpowerinfo->detres[1] * fitsstuff->egam + 
    stoppingpowerinfo->detres[2] * pow(fitsstuff->egam,2);
  g_return_if_fail(reso != 0);
  
  for ( i = 0; i < 160; i++) {
    gsq = pow(((float) i / ((reso * 6.006) / fitsstuff->ecal)) , 2);
    sumg = sumg + exp(-gsq);
  }
  
  sm = 1 + 2 * sumg;
  gn = (float) 1 / (float) sm;
  for ( i = 0; i < 160; i++) {
    j = i - 1;
    g = (float) j/ ((6.006 * reso) / fitsstuff->ecal);
    gsq = pow(g,2);
    gauss[i] = gn * exp(-gsq);
  }
  
  z123 = pow(stoppingpowerinfo->recz, 0.6666667);
  zt23 = pow(stoppingpowerinfo->targz, 0.6666667);
  zb23 = pow(stoppingpowerinfo->backz, 0.6666667);
  sqt = sqrt(z123 + zt23);
  sqb = sqrt(z123 + zb23);
  fitsstuff->epfac[0] = (32536 * stoppingpowerinfo->targa) / 
    (stoppingpowerinfo->recz * stoppingpowerinfo->targz * 
     (stoppingpowerinfo->targa + stoppingpowerinfo->reca) * sqt);
  fitsstuff->facnuke[0] =
    (5.1048 * stoppingpowerinfo->recz * stoppingpowerinfo->targz * stoppingpowerinfo->reca) /
    (stoppingpowerinfo->targa * sqt *
     (stoppingpowerinfo->targa * stoppingpowerinfo->reca));
  if (stoppingpowerinfo->tarstopnorm[1] == 0) {
    fitsstuff->epfac[1] = 0;
    fitsstuff->facnuke[1] = 0;
  } else {
    fitsstuff->epfac[1] = (32536 * stoppingpowerinfo->backa) / 
      (stoppingpowerinfo->recz * stoppingpowerinfo->backz * sqb *
       (stoppingpowerinfo->backa + stoppingpowerinfo->reca));
    fitsstuff->facnuke[1] = 
      (5.1048 * stoppingpowerinfo->recz * stoppingpowerinfo->backz * stoppingpowerinfo->reca) / 
      (stoppingpowerinfo->backa * sqb * (stoppingpowerinfo->backa + stoppingpowerinfo->reca));
  }

  if (stoppingpowerinfo->blauswitch) {
    eavg = (float) (stoppingpowerinfo->projmass * stoppingpowerinfo->beamenergy) /
      (float) (stoppingpowerinfo->projmass * stoppingpowerinfo->targa);
    if (stoppingpowerinfo->tarstopnorm[1] == 0) {
      aa2 = stoppingpowerinfo->targa;
      z2 = stoppingpowerinfo->targz;
    } else {
      aa2 = stoppingpowerinfo->backa;
      z2 = stoppingpowerinfo->backz;
    }
    
    /* --- this is where cobar is called
       --- I'm calling it FitsAvCos --- */
    FitsTauEntry(ar,zr,aa2,z2,eavg);
  }

  if ((stoppingpowerinfo->nrad == 1) && (stoppingpowerinfo->numrectheta == 1)) {
    fitsstuff->probn[0][0][0][0] = 1;
    stoppingpowerinfo->numrecphi = 1;
    stoppingpowerinfo->nang = 1;
    thetad = stoppingpowerinfo->theta * 3.1415926 / (float) 180;
    fitsstuff->cst[0][0][0][0] = cos(thetad);
  } else {
    /* --- this is where detgeom is called
       --- I'm going to call it DetectorGeomretry --- */
    DetectorGeometry();

    sumslt = 0;
    costh = cos(stoppingpowerinfo->theta / 57.296)
      for (i = 0; i < stoppingpowerinfo->nang; i++) {
	for ( j = 0; j < stoppingpowerinfo->nrad; j++) {
	  for ( k = 0; k < stoppingpowerinfo->numrecphi; k++) {
	    for (l = 0; l < stoppingpowerinfo->numrectheta; l++) {
	      sumslt = sumslt + fitsstuff->probn[i][j][l][k] *
	      pow((fitsstuff->cst[i][j][l][k] - costh),2);
	    }
	  }
	}
      }
    slt = sqrt(sumslt);
    angt = 57.296 * acos(slt);
  }
  fitsstuff->acn = stoppingpowerinfo->projmass + stoppingpowerinfo->targa;
  
  /* --- now we have egam and stop peak information --- */
  
  /* --- guess we'll try to calculate the spectrum --- */

  eltime = 0;
  for ( i = 0; i < 30; i++) {
    for ( j = 0; j < 1000; j++) {
      fitsstuff->specn[i][j] = 0;
    }
  }

  iflag = 0;
  ebeam = stoppingpowerinfo->beamenergy;

  /* --- this is where sigma is called --- */
  /* --- I call it CrossSection --- */
  sigmax = CrossSection(ebeam);
  for ( i = 0; i < stoppingpowerinfo->numtarel; i++) {
    ebi = ebeam;
    sigmr = CrossSection(ebi);
    if (sigmr < 0) goto crossdone;
    /* --- this is where kinemat is called --- */
    Kinemat(ebi,er);
    for ( j = 0; j < 7; j++) {
      fitsstuff->rnumi[j] = fitsstuff->eprob[j] * rnum * sigmr/sigmax;
    }
    jj = i;
    /* --- this is where shift is called --- */

  }
 crossdone:

  /* --- free appropriate memory --- */

  free(fitsstuff);

}

/* FitsTauEntry
 *
 * Interprets the tau entry from the dialog
 */
void FitsTauEntry(GtkWidget *widget, GtkWidget *entry)
{
  float a,b,c;
  int test;
  char dummystr[80];

  /* --- make sure that we get the right input --- */
  if (sscanf(gtk_entry_get_text(GTK_ENTRY(entry)),"%f %f %f",&a,&b,&c) == 3) {
    fitsstuff->taulo = a;
    fitsstuff->tauup = b;
    fitsstuff->dtau = c;
  } else {
    sprintf(dummystr,"Error reading tau parameters from entry.");
    GetMessageDialog(dummystr);
  }
}

/* FitsTauPrompt
 *
 * Prompts the user for the parameters for tau
 */
void FitsTauPrompt(GtkWidget *text)
{
  char dummystr[80];
  sprintf(dummystr,"Please enter Taulo Tauup, dTau.");
  gtk_text_insert(GTK_TEXT(text),NULL,NULL,NULL,dummystr,strnlen(&dummystr,80));
}

/* FitsAvCos
 *
 * Calculates the average cosine of multiple
 * scattering angle, cbar, due to nuclear scattering
 * of the recoils using Blaugrund's approx.
 */
void FitsAvCos(float a1, float z1, float a2, float z2, float er)
{
  float emax,arec,apa;
  int lphase;
  int i,j,k,l,m;
  float emax;
  float arec;
  float lphase;
  float apa,z123,z223;
  float r,g,am,d,ak;
  float dele,efac,delp,sumi,e,je,elp,dee,sqrel,den;

  emax = er;
  arec = a2;
  lphase = 1;
  if (emax > 5) emax = 5;
  apa = a1+a2;
  z123 = pow(z1,0.66667);
  z223 = pow(z2,0.66667);
  r = a1/a2;
  if ( r < 1) {
    g = 1 + 0.6666667 * r - 0.46667 * pow(r,2);
  } else {
    g = 0.6666667 + 8 / (15 * r);
  }
  am = 1630 * a1 * a2 / (z1 * z2 * sqrt(z123 + z223) * apa);
  d = pow((z123 + z223),0.75) * sqrt(pow (a1,3) * a2);
  ak = pow(z1,0.166667)*0.0793*sqrt(z1*z2*pow(apa,3)) / d;
  dele = 0.01;
  efac = 10.2 * am /(0.511 * a1);
  delp = dele * efac;
  sumi = 0;
  e = emax;
  je = 100 * e +0.5;
  while (je >= 0) {
    elp = e * efac;
    dee = ak *sqrt(elp);
    sqrel = - sqr(elp);
    den = 0.6111 * exp(sqrel/1.919) * (1 - exp(sqrel/0.406));
    sumi = sumi + delp * den / ((den + dee) * elp);
    fitsstuff->cbar[(int)je] = exp(-0.5*g*sumi/r);
    e = e - dele;
    je = 100 * e + 0.5;
  }

}

/* DetectorGeomretry
 *
 * This subroutine produces an array of gamma ray
 * detection probablilities, PROBN(ig), for each
 * detector element, and an array of gamma ray 
 * angles through each detector element CST.  PROBN
 * and CST are also indexed by theta and phi of the
 * recoil direction.
 */
void DetectorGeometry()
{
  float thetad;
  float ssd,csd,rho1,delphi,delrho;
  float delphir,sump,rho,rhsq,dsq,resq,ael,domega,xel;
  float rd,xd,shot,arg,eff,phi,csi,ssi,xp,yp,zp;
  float xp2,yp2,zp2,densq,den,thetr;
  float ssr,csr,phir,xr,yr,zr,csp;
  float p2,p4,wt;
  int i,j,k,l;

  thetd = stoppingpowerinfo->theta * 3.1415926 / 180;
  ssd = sin(thetd);
  cst = cos(thetd);
  rho1 = stoppingpowerinfo->r * stoppingpowerinfo->d / 
    (stoppingpowerinfo->r + stoppingpowerinfo->d);
  delphi = 3.1415926 / (float) stoppingpowerinfo->nang;
  delrho = stoppingpowerinfo->r / stoppingpowerinfo->nrad;
  delphir = 6.2831853;  // 2 * pi
  
  sump = 0;
  for (i = 0; i < stoppingpowerinfo->nrad; i++) {
    rho = ((float)i - 0.5) * delrho;
    rhsq = pow(rho,2);
    dsq = pow(stoppingpowerinfo->d,2);
    relsq = dsq + rhsq;
    ael = 0.5 * (2 * rho + delrho) * delrho * delphi;
    domega = ael / relsq;
    if (rho < rho1) {
      xel = stoppingpowerinfo->h / stoppingpowerinfo->d * sqrt(relsq);
    } else {
      rd = pow( (stoppingpowerinfo->r - rho),2);
      xd = 1 + dsq / rhsq;
      shot = rd * xd;
      xel = sqrt(shot);
    }
    arg = 1000 & xel / fitsstuff->egam;
    eff = 1 - exp(-arg);
    for ( j = 0; j < stoppingpowerinfo->nang; j++) {
      phi = ((float)j - 0.5) * delphi;
      csi = cos(phi);
      ssi = sin(phi);
      xp = rho * csi * csd + stoppingpowerinfo->d * ssd;
      yp = rho * ssi;
      zp = stoppingpowerinfo->d * csd - rho * csi * ssd;
      xp2 = pow(xp,2);
      yp2 = pow(yp,2);
      zp2 = pow(zp,2);
      densq = xp2 + yp2 + zp2;
      den = sqrt(densq);
      xp = xp / den;
      yp = yp / den;
      zp = zp / den;
      /* --- xp,yp,zp is a unit vector pointing from the target to the current
	 --- detector segment --- */
      for (k = 0; k < stoppingpowerinfo->numrectheta; k++) {
	thetr = stoppingpowerinfo->rectheta[k];
	ssr = sin(thetr);
	csr = cos(thetr);
	for ( l = 0; l < stoppingpowerinfo->numrecphi; l++) {

	  phir = ((float) l - 0.5) * delphir;
	  xr = ssr * sin(phir);
	  yr = ssr * cos(phir);
	  zr = csr;

	  /* --- xr,yr,zr is a unit vector in the direction or the recoil
	     --- velocity.  The cos of the angle between the detector
	     --- and the recoil directions is given by the
	     --- dot product. --- */
	  csp = xr * xp + yr * yp + zr * zp;
	  p2 = AP2(csp);
	  p4 = AP4(csp);
	  wt = 1 + stoppintpowerinfo->a2 * p2 + stoppingpowerinfo->a4 * p4;
	  fitsstuff->cst[j][i][k][l] = csp;
	  fitsstuff->probn[j][i][k][l] = domega * eff * wt * 
	    stoppingpowerinfo->recprobtheta[k];
	  sump = sump + fitsstuff->probn[j][i][k][l];
	} // done l loop

      } // done k loop

    } // done j loop

  } // done i loop


  for (i = 0; i < stoppingpowerinfo->nrad; i++) {  
    for ( j = 0; j < stoppingpowerinfo->nang; j++) {
      for (k = 0; k < stoppingpowerinfo->numrectheta; k++) {
	for ( l = 0; l < stoppingpowerinfo->numrecphi; l++) {
	  fitsstuff->probn[j][i][k][l] = fitsstuff->probn[j][i][k][l] / sump;
	}
      }
    }
  }
}

/* AP2
 * short function used in Detector Geometry
 */
float AP2(float x)
{
  return(0.5 * (3 * pow(x,2) - 1));
}

/* AP4
 * short function used in Detector Geometry
 */
float AP4(float x)
{
  return(0.125 * (35 * pow(x,4) - 30 * pow(x,2) + 3));
}

/* CrossSection
 *
 * This subroutine calculates the residual nucleus
 * production cross-section for the input
 * energy by searching for the nearest
 * energy entries in the input tables, and 
 * performing a linear interpolation to obtain
 * the value for crossection
 */
float CrossSection(float beamenergy)
{

  int i,j,k,l,m;
  float elo,eup,sigma;
  int jlo,jup;

  for (i = 0; i < stoppingpowerinfo->numresidprodcross; i++) {
    jlo = i;
    jup = i+1;
    elo = stoppingpowerinfo->crosssectionen[jlo];
    ehi = stoppingpowerinfo->crosssectionen[jup];
    if (beamenergy == elo) {
      return(stoppingpowerinfo->crosssection[jlo]);
    }
    if (beamenergy == ehi) {
      return(stoppingpowerinfo->crosssection[jhi]);
    }
    if ((beamenergy > elo) && ( beamenergy < ehi)) {
      /* --- twint is called, I call it LinearInterPol --- */
      return(LinearInterPol(elo,eup,stoppingpowerinfo->crosssection[jlo],
			    stoppingpowerinfo->crosssection[jhi],beamenergy));
    }
  }
}

/* LinearInterPol
 *
 * Returns a float to the y value of a linear interpolation
 * between two points for a given x
 */
float LinearInterPol(float x1, float x2, float y1, float y2, float x)
{
  float det,anum,bnum,deno;
  float a,b,c;

  deno = x1 - x2;
  anum = y1 * x2 - y2 * x2;
  bnum = y1 - y2;
  a = anum / deno;
  b = bnum / deno;
  return(a + b * x);
}

/* Kinemat
 *
 * This subroutine calculates the energy of the
 * recoiling nuclei, ER, for an input projectile
 * energy
 */
void Kinemat(float beamenergy, float recoilenergy)
{
  float ecn,betacn,eavg,sig,arg,argsq,ter,sum,earg,eargsq,eterm;
  int i,j,k;
  
  ecn = beamenergy * stoppingpowerinfo->projmass / fitsstuff->acn;
  betacn = sqrt((2*ecn) / (931.5 * acn));
  eavg = 0.5 * stoppingpowerinfo->reca * 931.5 * pow(betacn,2);
  sig = del * eavg/1.4142;
  for ( i = 0; i < 7; i++) {
    fitsstuff->er[i] = eavg + (4 - i) * sig;
  }
  sum = 0;
  for (i = 0; i < 3; i++) {
    arg = (fitsstuff->er[i] - eavg) / (1.4142 * sig);
    argsq = pow(arg,2);
    term = exp(-argsq);
    sum = sum + term;
  }
  pnorm = 1 / ( 1 + 2 * sum);
  for ( i = 0; i < 7; i++) {
    earg = (fitsstuff->er[i] - eavg) / ( 1.4142 * sig);
    eargsq = pow(earg,2);
    eterm = exp(-eargsq);
    fitsstuff->eprob[i] = pnorm * eterm;
  }
}

/* Shift
 *
 * Takes the recoils produced in the jth target element 
 * with energy enow, and propagates them through the 
 * remianing target elmenets.  It returns two shifted 
 * spectra, spec without detector resolution folded in 
 * and spec (ig,500) = the unshifted peaks.
 */
void Shift(int ig,float enw, int jj, float delx)
{
  int i,j,k;
  float a,b,c;
  float tt;
  int flag;
  int kstop;
  float enow[7];

  enow[0] = enw;

  tt = 0;
  for ( i = 0; i < 30; i++) {
    for (j = 0; j < 1000; j++) {
      fitsstuff->spec[i][j];
    }
  }
  
  /* --- loop through target elements --- */
  for ( i = 0; i < 7; i++) {
    rnumir = fitsstuff->rnumi[i];
    flag = 0;
    kstop = 0;
    for ( j = 0; j < 20000; j++) {
      
      ei = enow[i];

      /* --- check to see if the recoils have stopped --- */
      if (ei <= 0.001) {
	flag = 2;
	/* --- this is where depop is used 
	   --- I call it DepopulateLevels --- */
      } 

    } // done j loop

  } // done i loop

}

/* DepopulateLevels
 *
 * This subroutine sets up initial populations
 * of each level, then depopulates them
 * using the radioactive decay law, 
 * dn = n * time/tau, for each level.  It then
 * puts the decays from each level into
 * the level below it, and finally calculates
 * the number of decays from each of the
 * levels of interest and returns the values 
 * in dki
 */
void DepopulateLevels(float time, int flag)
{
  float dnn;
  int i,j,k;
  float a,b,c;
  float sfi[4][10];
  float dn[4][11];
  float anumi[4][10];
  float anum2[30];

  dnn = 0;
  if ( flag == 0) {
    /* --- set up initial populations --- */

    for (i = 0; i < 10; i ++) {
      k = 9 - i;
      if (k > 1) {
	for (j = 0; j < 4; j++) {
	  sfi[j][k] = fitsstuff->pop[j][k]; * fitsstuff->rnumi[0] / fitsstuff->pop[0][0];
	} // done with j loop
      } else {
	sfi[j][k] = fitsstuff->rnumi[0] * 
	  (fitsstuff->pop[j][k] - fitsstuff->pop[j][k+1]) / fitsstuff->pop[0][0];
      }
    } // done with i loop
    sfi[1][0] = (fitsstuff->pop[1][0] - fitsstuff->pop[2][0] - fitsstuff->pop[2][1]) *
      fitsstuff->rnumi[0] / fitsstuff->pop[0][0];
    sfi[1][1] = (fitsstuff->pop[1][1] - fitsstuff->pop[2][2] - fitsstuff->pop[2][3]) * 
      fitsstuff->rnumi[0] / fitsstuff->pop[0][0];
    sfi[0][0] = (fitsstuff->pop[0][0] - fitsstuff->pop[1][0] - fitsstuff->pop[1][1]) *
      fitsstuff->rnumi[0] / fitsstuff->pop[0][0];
    if (fitsstuff->pop[0][1] > 0) {
      sfi[0][0] = (fitsstuff->pop[0][0] - fitsstuff->pop[0][1]) * rnumi / fitsstuff->pop[0][0];
    }
    for ( i = 0; i < 4; i++) {
      dn[i][10] = 0;
      for ( j = 0; j < 10; j++) {
	dn[i][j] = 0;
	anumi[i][j] = 0;
      }
    }
    for ( i = 0; i < 30; i++) anum2[i] = 0;

  } // done with flag == 0

  /* --- now we have the initial populations, calculate the number
     --- of decays from each level, ignoring levels that have
     --- already decayed away. --- */
  sf = 0;
  for ( i = 0; i < 4; i++) {
    for ( k = 2; k < 10; k++) {
      j = 11 - k;
      if (sfi[i][j] < 0) {
	sf = 0;
      } else {
	if (fitsstuff->tsf[i][j] > 0) {
	  sf = sfi[i][j] * ( 1 - exp(-time / fitsstuff->tsf[i][j]));
	  sfi[i][j] = sfi[i][j] - sf;
	  if (sfi[i][j] < 0) sfi[i][j] = 0;
	} else {
	  sf = sfi[i][j];
	  sfi[i][j] = 0;
	}
      }
      anumi[i][j] = anumi[i][j] + sf + dn[i][j+1];
      if (tau[i][j] > 0) {
	dn[i][j] = anumi[i][j] * (1 - exp(-time/fitsstuff->tau[i][j]));
	anumi[i][j] = anumi[i][j] - dn[i][j];
	if (anumi[i][j] < 0) anumi[i][j] = 0;
      } else {
	dn[i][j] = anumi[i][j];
	anumi[i][j] = 0;
      }
    } // done with k loop
    dnc[i] = dn[2][i];
  }  // done with i loop
  
  /* --- finished decaying the highest cascade chains, dn[j] j = 1,4
     --- are the # of decays from each chain. --- */

  
  
}
